import os
import json

from dateutil.parser import parse


class Resource(object):
    __readonly__ = ['id', 'createdBy', 'modifiedBy', 'createdDate', 'modifiedDate']
    __subresources__ = []

    def __init__(self, d, api):
        self._api = api
        self._json = d

    def _path(self, *comps):
        _comps = ['/%ss' % self.__class__.__name__.lower()]
        if self._json.get('id'):
            _comps.append(self.id)
        return '/'.join(_comps + list(comps))

    def __getattr__(self, attr):
        """Attribute lookup.

        Attributes provide easy access to the following things:

        - subresources,
        - top-level keys in the resource's JSON representation.
        """
        if attr in self.__subresources__:
            # we fetch subresources using an API call when the corresponding attribute
            # is accessed - every time.
            json = True if not isinstance(self.__subresources__, dict) \
                else self.__subresources__[attr]
            return self._api._req(self._path() + '/' + attr, json=json)
        try:
            res = self._json[attr]
        except KeyError:
            raise AttributeError(attr)

        if attr.endswith('Date'):
            res = parse(res)

        #
        # TODO: once this is available, resolve users to proper objects upon access!
        #

        return res

    def __setattr__(self, attr, value):
        if attr.startswith('_'):
            return object.__setattr__(self, attr, value)
        if attr in self.__readonly__:
            raise AttributeError('%s is readonly' % attr)
        self._json[attr] = value

    def dumps(self, **kw):
        return json.dumps(self._json, **kw)

    def __repr__(self):
        return self.dumps(sort_keys=True, indent=4, separators=(',', ': '))

    def save(self):
        kw = dict(
            method='put' if self._json.get('id') else 'post',
            data=self.dumps(),
            headers={'content-type': 'application/json'})
        if kw['method'] == 'post':
            kw['assert_status'] = 201
        return self.__class__(self._api._req(self._path(), **kw), self._api)

    def delete(self):
        return self._api._req(
            self._path(), method='delete', assert_status=204, json=False)


class Collection(Resource):
    __subresources__ = ['mdprofiles']

    def add_item(self, **kw):
        return self._api.create('item', collectionId=self.id, **kw)

    def release(self):
        return self._api._req(
            self._path('release'), method='put', assert_status=204, json=False)


class Profile(Resource):
    pass


class Item(Resource):
    __subresources__ = {'content': False}

    def __init__(self, d, api):
        self.__file = None
        _file = d.pop('_file', None)
        if _file:
            setattr(self, '_file', _file)
        Resource.__init__(self, d, api)

    @property
    def _file(self):
        return self.__file

    @_file.setter
    def _file(self, value):
        assert os.path.exists(value)
        self.__file = value

    def save(self):
        # FIXME: verify md5 sum upon creation of item from local file!
        return self.__class__(
            self._api._req(
                self._path(),
                method='put' if self._json.get('id') else 'post',
                assert_status=200 if self._json.get('id') else 201,
                files=dict(
                    file=open(self._file, 'rb') if self._file else '',
                    json=self.dumps())),
            self._api)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""imeji

Usage:
  imeji create <what> <properties>
  imeji [options] retrieve <what> <id>
  imeji delete <what> <id>
  imeji -h | --help
  imeji --version

Options:
  -h --help        Show this screen.
  --version        Show version.
  --service=<URL>  URL of the imeji service
"""

from __future__ import unicode_literals, print_function
import sys

from docopt import docopt

from pyimeji.config import Config
from pyimeji.api import Imeji


__version__ = "0.1.1"
__author__ = "Robert Forkel"
__license__ = "MIT"


def parsed_kw(s):
    return {k: v for k, v in [pair.split('=') for pair in s.split(';')]}


def checked_call(f, *args, **kw):
    try:
        return f(*args, **kw)
    except AssertionError:
        return Exception()


def main(argv=None):  # pragma: no cover
    """Main entry point for the imeji CLI."""
    cfg = Config()
    args = docopt(__doc__, version=__version__, argv=argv)
    api = Imeji(cfg, service_url=args['--service'])
    if args['retrieve']:
        return checked_call(getattr(api, args['<what>']), id=args['<id>'])
    if args['create']:
        return checked_call(api.create, args['<what>'], **parsed_kw(args['<properties>']))
    if args['delete']:
        return checked_call(
            api.delete, checked_call(getattr(api, args['<what>']), id=args['<id>']))


if __name__ == '__main__':  # pragma: no cover
    res = main()
    if isinstance(res, Exception):
        status = -1
    else:
        status = 0
        print(res)
    sys.exit(status)



# coding=utf8
from __future__ import unicode_literals
from unittest import TestCase
from datetime import datetime

from httmock import all_requests, response, HTTMock


SERVICE_URL = 'http://example.org'


class Response(object):
    def __init__(self, key, status, content):
        self.key = key
        self.status = status
        self.content = content


RESPONSES = [
    Response(('/rest/items', 'get'), 200, {"Wo1JI_oZNyrfxV_t": {}}),
    Response(('/rest/items/Wo1JI_oZNyrfxV_t', 'delete'), 204, {}),
    Response(('/rest/items/Wo1JI_oZNyrfxV_t', 'get'), 200, {
        "id": "Wo1JI_oZNyrfxV_t",
        "createdBy": {
            "fullname": "Saquet",
            "id": "zhcQKsMR0A9SiC6x"
        },
        "modifiedBy": {
            "fullname": "Saquet",
            "id": "zhcQKsMR0A9SiC6x"
        },
        "createdDate": "2014-10-16T11:01:13 +0200",
        "modifiedDate": "2014-11-20T10:31:15 +0100",
        "versionDate": "2014-10-16T11:15:97 +0200",
        "status": "RELEASED",
        "version": 1,
        "discardComment": "",
        "visibility": "PUBLIC",
        "collectionId": "FKMxUpYdV9N2J4XG",
        "filename": "virr-image.tif",
        "mimetype": "application/octet-stream",
        "checksumMd5": "006a6dc402c6d84dc4c10d955599f57c",
        "webResolutionUrlUrl": "http://example.org/image.jpg",
        "thumbnailUrl": "http://example.org/image.jpg",
        "fileUrl": "http://example.org/image.jpg",
        "metadata": [
            {
                "position": 0,
                "value": {
                    "text": "title"
                },
                "statementUri": "http://imeji.org/terms/statement/xx9HntRF7GuFIJA",
                "typeUri": "http://imeji.org/terms/metadata#text",
                "labels": [
                    {
                        "language": "en",
                        "value": "titel"
                    }
                ]
            },
            {
                "position": 1,
                "value": {
                    "name": "Munich, Germany",
                    "longitude": 11.5819806,
                    "latitude": 48.1351253
                },
                "statementUri": "http://imeji.org/terms/statement/dCIHVzo4TWBD1bWc",
                "typeUri": "http://imeji.org/terms/metadata#geolocation",
                "labels": [
                    {
                        "language": "en",
                        "value": "goelocation"
                    }
                ]
            },
            {
                "position": 2,
                "value": {
                    "person": {
                        "position": 0,
                        "id": "sCooi0G_D8SW5__",
                        "familyName": "Saquet",
                        "givenName": "Bastien",
                        "completeName": "Saquet, Bastien",
                        "alternativeName": "",
                        "role": "author",
                        "identifiers": [
                            {
                                "type": "imeji",
                                "value": "http://pubman.mpdl.mpg.de/cone/persons"
                                         "/resource/persons96343"
                            }
                        ],
                        "organizations": [
                            {
                                "position": 0,
                                "id": "KHUV7faLBp9R5Zqm",
                                "name": "Innovations, MPDL, Max Planck Society",
                                "description": "",
                                "identifiers": [
                                    {
                                        "type": "imeji",
                                        "value": "escidoc:persistent1"
                                    }
                                ],
                                "city": "",
                                "country": ""
                            }
                        ]
                    }
                },
                "statementUri": "http://example.org/statement",
                "typeUri": "http://imeji.org/terms/metadata#conePerson",
                "labels": [
                    {
                        "language": "en",
                        "value": "Person"
                    }
                ]
            }
        ]
    }),
    Response(('/rest/items/Wo1JI_oZNyrfxV_t/content', 'get'), 200, b'test'),
    Response(('/rest/collections', 'get'), 200, {"FKMxUpYdV9N2J4XG": {}}),
    Response(('/rest/collections/FKMxUpYdV9N2J4XG/release', 'put'), 204, {}),
    Response(('/rest/collections/FKMxUpYdV9N2J4XG', 'delete'), 204, {}),
    Response(('/rest/collections/FKMxUpYdV9N2J4XG', 'get'), 200, {
        "id": "FKMxUpYdV9N2J4XG",
        "createdBy": {
            "fullname": "Saquet",
            "id": "zhcQKsMR0A9SiC6x"
        },
        "modifiedBy": {
            "fullname": "admin",
            "id": "w7ZfmmC5LQR8KJIN"
        },
        "createdDate": "2014-10-09T13:01:25 +0200",
        "modifiedDate": "2014-11-19T14:50:21 +0100",
        "versionDate": "2014-10-16T11:15:44 +0200",
        "status": "RELEASED",
        "version": 1,
        "discardComment": "",
        "title": "Research Data",
        "description": "Test for research data",
        "contributors": [
            {
                "position": 0,
                "id": "Od6MTBSdXYPWYSU1",
                "familyName": "Saquet",
                "givenName": "Bastien",
                "completeName": "Saquet, Bastien",
                "alternativeName": "",
                "role": "author",
                "identifiers": [
                    {
                        "type": "imeji",
                        "value": "http://pubman.mpdl.mpg.de/cone/persons"
                                 "/resource/persons96343"
                    }
                ],
                "organizations": [
                    {
                        "position": 0,
                        "id": "94zMk2OtjJAWtXIH",
                        "name": "Innovations, MPDL, Max Planck Society",
                        "description": "",
                        "identifiers": [
                            {
                                "type": "imeji",
                                "value": "escidoc:persistent1"
                            }
                        ],
                        "city": "",
                        "country": ""
                    }
                ]
            },
            {
                "position": 0,
                "id": "4S34HiEj6qvvgTV1",
                "familyName": "Bulatovic",
                "givenName": "Natasa",
                "completeName": "Bulatovic, Natasa",
                "alternativeName": "",
                "role": "author",
                "identifiers": [
                    {
                        "type": "imeji",
                        "value": "http://pubman.mpdl.mpg.de/cone/persons"
                                 "/resource/persons96308"
                    }
                ],
                "organizations": [
                    {
                        "position": 0,
                        "id": "OSCfm3llkE3Foa4",
                        "name": "Innovations, MPDL, Max Planck Society",
                        "description": "",
                        "identifiers": [
                            {
                                "type": "imeji",
                                "value": "escidoc:persistent1"
                            }
                        ],
                        "city": "",
                        "country": ""
                    }
                ]
            }
        ],
        "profileId": "dhV6XK39_UPrItK5"
    }),
    Response(('/rest/profiles/dhV6XK39_UPrItK5', 'get'), 200, {
        "id": "dhV6XK39_UPrItK5",
        "createdBy": {
            "fullname": "Saquet, Bastien",
            "userId": "zhcQKsMR0A9SiC6x"
        },
        "modifiedBy": {
            "fullname": "Saquet, Bastien",
            "userId": "zhcQKsMR0A9SiC6x"
        },
        "createdDate": "2014-10-09T13:01:49 +0200",
        "modifiedDate": "2014-11-20T10:31:13 +0100",
        "versionDate": "2014-10-16T11:15:54 +0200",
        "status": "RELEASED",
        "version": 0,
        "discardComment": "",
        "title": "Research Data",
        "description": "Test for research data",
        "statements": [
            {
                "id": "xx9HntRF7GuFIJA",
                "pos": 0,
                "type": "http://imeji.org/terms/metadata#text",
                "labels": [
                    {
                        "value": "titel",
                        "lang": "en"
                    }
                ],
                "vocabulary": None,
                "literalConstraints": [],
                "minOccurs": "0",
                "maxOccurs": "1",
                "parentStatementId": None,
                "useInPreview": True
            },
        ]
    }),
]
RESPONSES = {r.key: r for r in RESPONSES}

RESPONSES[('/rest/collections', 'post')] = Response(
    ('/rest/collections', 'post'),
    201,
    RESPONSES[('/rest/collections/FKMxUpYdV9N2J4XG', 'get')].content)
RESPONSES[('/rest/items/Wo1JI_oZNyrfxV_t', 'put')] = Response(
    ('/rest/items', 'post'),
    200,
    RESPONSES[('/rest/items/Wo1JI_oZNyrfxV_t', 'get')].content)
RESPONSES[('/rest/items', 'post')] = Response(
    ('/rest/items', 'post'),
    201,
    RESPONSES[('/rest/items/Wo1JI_oZNyrfxV_t', 'get')].content)


@all_requests
def imeji(url, request):
    headers = {'content-type': 'application/json'}
    res = RESPONSES[(url.path, request.method.lower())]
    return response(res.status, res.content, headers, None, 5, request)


class ApiTest(TestCase):
    def setUp(self):
        from pyimeji.api import Imeji

        self.api = Imeji(service_url=SERVICE_URL)

    def test_collection(self):
        with HTTMock(imeji):
            collections = self.api.collections()
            self.assertIn('FKMxUpYdV9N2J4XG', collections)
            collection = self.api.collection(list(collections.keys())[0])
            self.assertEqual(collection.title, 'Research Data')
            collection.title = 'New title'
            self.assertIsInstance(collection.createdDate, datetime)
            self.assertEqual(collection.title, 'New title')
            collection.dumps()
            repr(collection)
            collection2 = self.api.create('collection', title='abc')
            collection2.add_item(referenceUrl='http://example.org/')
            assert collection2
            collection.release()
            collection2.delete()

    def test_item(self):
        with HTTMock(imeji):
            self.api.create('item', _file=__file__)
            items = self.api.items()
            self.assertIn('Wo1JI_oZNyrfxV_t', items)
            item = self.api.item(list(items.keys())[0])
            self.assertEqual(item.filename, 'virr-image.tif')
            self.assertRaises(ValueError, self.api.item)
            self.assertRaises(AttributeError, getattr, item, 'abc')
            self.assertRaises(AttributeError, setattr, item, 'id', 'abc')
            self.assertEqual(item.content.text, 'test')
            item2 = self.api.create('item', collectionId='abc', file=__file__)
            assert item2
            self.api.update(item2, filename='name.png')
            item2.delete()
            self.api.delete(item)

    def test_profile(self):
        with HTTMock(imeji):
            profile = self.api.profile('dhV6XK39_UPrItK5')
            self.assertEqual(profile.title, 'Research Data')

    def test_cli(self):
        from pyimeji.cli import main
        from pyimeji.resource import Collection

        with HTTMock(imeji):
            res = main(
                ('--service=%s retrieve collection FKMxUpYdV9N2J4XG' % SERVICE_URL)
                .split())
            self.assertIsInstance(res, Collection)



from __future__ import unicode_literals
import os
from tempfile import mkdtemp
from unittest import TestCase
from shutil import rmtree
from io import open


class ConfigTest(TestCase):
    def setUp(self):
        self.tmp = mkdtemp()
        self.cfg = os.path.join(self.tmp, '.config')

    def tearDown(self):
        rmtree(self.tmp, ignore_errors=True)

    def test_new_config(self):
        from pyimeji.config import Config

        self.assertFalse(os.path.exists(self.cfg))
        Config(config_dir=self.cfg)
        self.assertTrue(os.path.exists(self.cfg))

    def test_existing_config(self):
        from pyimeji.config import Config

        os.mkdir(self.cfg)
        with open(os.path.join(self.cfg, 'config.ini'), 'w', encoding='utf8') as fp:
            fp.write("""\
[section]
option = 12
""")

        cfg = Config(config_dir=self.cfg)
        self.assertEqual(cfg.get('section', 'option'), '12')



# -*- coding: utf-8 -*-
import re
import sys
from setuptools import setup, find_packages


requires = [
    'six',
    'docopt',
    'requests',
    'AppDirs',
    'python-dateutil',
]


def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content


setup(
    name='pyimeji',
    version="0.2.0",
    description='A python wrapper for the imeji API',
    long_description=read("README.rst"),
    author='Robert Forkel',
    author_email='xrotwang@googlemail.com',
    url='https://github.com/imeji-community/pyimeji',
    install_requires=requires,
    license=read("LICENSE"),
    zip_safe=False,
    keywords='imeji',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            "imeji = pyimeji.cli:main"
        ]
    },
    tests_require=['nose', 'coverage', 'httmock'],
)



import os
import logging

from appdirs import AppDirs
from six.moves.configparser import RawConfigParser


APP_DIRS = AppDirs('pyimeji')


class NoDefault(object):
    pass

NO_DEFAULT = NoDefault()


class Config(RawConfigParser):
    def __init__(self, **kw):
        config_dir = kw.pop('config_dir', None) or APP_DIRS.user_config_dir
        RawConfigParser.__init__(self, kw)

        cfg_path = os.path.join(config_dir, 'config.ini')
        if os.path.exists(cfg_path):
            assert os.path.isfile(cfg_path)
            self.read(cfg_path)
        else:
            if not os.path.exists(config_dir):
                try:
                    os.mkdir(config_dir)
                except OSError:  # pragma: no cover
                    # this happens when run on travis-ci, by a system user.
                    pass
            if os.path.exists(config_dir):
                with open(cfg_path, 'w') as fp:
                    self.write(fp)
        level = self.get('logging', 'level', default=None)
        if level:
            logging.basicConfig(level=getattr(logging, level))

    def get(self, section, option, default=NO_DEFAULT):
        if default is not NO_DEFAULT:
            if not self.has_option(section, option):
                return default
        return RawConfigParser.get(self, section, option)



"""A client for the REST API of imeji instances."""
import logging

import requests
from six import string_types

from pyimeji import resource
from pyimeji.config import Config


log = logging.getLogger(__name__)


class GET(object):
    """Handle GET requests.

    This includes requests

    - to retrieve single objects,
    - to fetch lists of object references.
    """
    def __init__(self, api, name):
        """Initialize a handler.

        :param api: An Imeji API instance.
        :param name: Name specifying the kind of object(s) to retrieve. We check whether\
        this name has a plural "s" to determine if a list is to be retrieved.
        """
        self._list = name.endswith('s')
        self.rsc = getattr(resource, (name[:-1] if self._list else name).capitalize())
        self.api = api
        self.name = name
        self.path = name
        if not self._list:
            self.path += 's'

    def __call__(self, id=''):
        """Calling the handler initiates an HTTP request to the imeji server.

        :param id: If a single object is to be retrieved it must be specified by id.
        :return: The pure JSON response for lists, a \
        :py:class:`pyimeji.resource.Resource` instance for single objects.
        """
        if not self._list and not id:
            raise ValueError('no id given')
        if id:
            id = '/' + id
        res = self.api._req('/%s%s' % (self.path, id))
        if not self._list:
            res = self.rsc(res, self.api)
        return res


class Imeji(object):
    """The client.

    >>> api = Imeji(service_url='http://demo.imeji.org/imeji/')
    >>> collection_id = list(api.collections().keys())[0]
    >>> collection = api.collection(collection_id)
    >>> collection = api.create('collection', title='the new collection')
    >>> item = collection.add_item(fetchUrl='http://example.org')
    >>> item.delete()
    """
    def __init__(self, cfg=None, service_url=None):
        self.cfg = cfg or Config()
        self.service_url = service_url or self.cfg.get('service', 'url')
        user = self.cfg.get('service', 'user', default=None)
        password = self.cfg.get('service', 'password', default=None)
        self.session = requests.Session()
        if user and password:
            self.session.auth = (user, password)

    def _req(self, path, method='get', json=True, assert_status=200, **kw):
        method = getattr(self.session, method.lower())
        res = method(self.service_url + '/rest' + path, **kw)
        if assert_status:
            if res.status_code != assert_status:  # pragma: no cover
                log.error(
                    'got HTTP %s, expected HTTP %s' % (res.status_code, assert_status))
                log.error(res.text[:1000])
                raise AssertionError()
        if json:
            try:
                res = res.json()
            except ValueError:  # pragma: no cover
                log.error(res.text[:1000])
                pass
        return res

    def __getattr__(self, name):
        return GET(self, name)

    def create(self, rsc, **kw):
        if isinstance(rsc, string_types):
            cls = getattr(resource, rsc.capitalize())
            rsc = cls(kw, self)
        return rsc.save()

    def delete(self, rsc):
        return rsc.delete()

    def update(self, rsc, **kw):
        for k, v in kw.items():
            setattr(rsc, k, v)
        return rsc.save()



from __future__ import unicode_literals
from unittest import TestCase


class CliTest(TestCase):
    def test_parsed_kw(self):
        from pyimeji.cli import parsed_kw

        parsed = parsed_kw('a=2;b=3')
        self.assertEquals(parsed['a'], '2')

    def test_checked_call(self):
        from pyimeji.cli import checked_call

        def f1(a, b=3):
            return a + b

        def f2(exc=AssertionError):
            raise exc()

        self.assertEquals(checked_call(f1, 3), 6)
        self.assertIsInstance(checked_call(f2), Exception)
        self.assertRaises(ValueError, checked_call, f2, ValueError)
